import logging

from trosnoth.messages.base import AgentRequest, ServerCommand

log = logging.getLogger()


class PlayerHasElephantMsg(ServerCommand):
    idString = 'jken'
    fields = 'playerId'
    packspec = 'c'


class PlayerHasTrosballMsg(ServerCommand):
    idString = 'ball'
    fields = 'playerId'
    packspec = 'c'


class TrosballPositionMsg(ServerCommand):
    idString = 'bing'
    fields = 'xpos', 'ypos', 'xvel', 'yvel'
    packspec = 'ffff'


class ThrowTrosballMsg(AgentRequest):
    idString = 'pass'
    fields = 'tickId'
    packspec = 'H'

    def clientValidate(self, localState, world, sendResponse):
        if not localState.player:
            return False
        if localState.player.id != world.trosballPlayer.id:
            return False
        return True

    def serverApply(self, game, agent):
        if agent.player and agent.player == game.world.trosballPlayer:
            game.world.throwTrosball()


class TrosballTagMsg(ServerCommand):
    idString = 'goal'
    fields = 'teamId', 'playerId'
    packspec = 'cc'


class AchievementUnlockedMsg(ServerCommand):
    idString = 'Achm'
    fields = 'playerId', 'achievementId'
    packspec = 'c*'


class UpdateClockStateMsg(ServerCommand):
    idString = 'clok'
    fields = 'showing', 'counting', 'upwards', 'value', 'flashBelow'
    packspec = 'bbbff'

    def applyOrderToWorld(self, world):
        if world.isServer:
            # These messages are generated by World.clock.propagateToClients,
            # based off the existing state of the clock, so we don't need to
            #  set it again on the server.
            return
        world.clock.value = self.value
        world.clock.flashBelow = self.flashBelow
        world.clock.setMode(
            showing=self.showing,
            counting=self.counting,
            upwards=self.upwards,
        )


class PlaySoundMsg(ServerCommand):
    idString = 'noyz'
    fields = 'filename'
    packspec = '*'